#include <stdlib.h>
#include <stdio.h>

#include "openSE.h"
#include "resource.h"
#include "COmplexNumbers.h"

#define SURF_WIDTH		320
#define SURF_HEIGHT		200

class CExample5 : public CApplication
{
public:
	openSE*			surface;
	dword_*			buffer;
	HWND			iterChild;
	HWND			timeChild;

	CExample5() : CApplication()
	{
		m_width		= 640;
		m_height	= 480;
		m_name		= string( "Example5 - Fractals" );

		surface		= NULL;
		buffer		= NULL;
		iterChild	= NULL;
		timeChild	= NULL;
	}
	~CExample5() {}

	virtual void	DoFrame( double _timeDelta );
	virtual void	SceneInit();
	virtual void	SceneEnd();

	double			MakeMandelBrot( dword_* surf, int max_iterations );

	int				GetRadioButton();
	void			SetTime( double _time );

};

CApplication* CApplication::ApplicationCreate()
{
	return new CExample5;
}

void CApplication::ApplicationDestroy()
{
	delete this;
}

void CExample5::SceneInit()
{
	// Creates child windows
    timeChild = CreateDialog( GetInstance(), MAKEINTRESOURCE( IDD_TIME ), GetHWND(), NULL );
	assert( timeChild );
	ShowWindow( timeChild, SW_SHOW );
    iterChild = CreateDialog( GetInstance(), MAKEINTRESOURCE( IDD_ITER ), GetHWND(), NULL );
	assert( iterChild );
	ShowWindow( iterChild, SW_SHOW );

	surface = new openSE;
	assert( surface );

	buffer = surface->Open( SURF_WIDTH, SURF_HEIGHT, GetDD() );
	assert( buffer );

	double time = MakeMandelBrot( buffer, 70 );
	SetTime( time );
	SendMessage( GetDlgItem( iterChild, IDC_TYP2 ), BM_SETCHECK, BST_CHECKED, 0 );
}

void CExample5::SceneEnd()
{
	DestroyWindow( timeChild );
	DestroyWindow( iterChild );

	delete surface;
}

int CExample5::GetRadioButton()
{
	if( SendMessage( GetDlgItem( iterChild, IDC_TYP1 ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		return 30;
	else if( SendMessage( GetDlgItem( iterChild, IDC_TYP2 ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		return 70;
	else if( SendMessage( GetDlgItem( iterChild, IDC_TYP3 ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		return 100;
	else if( SendMessage( GetDlgItem( iterChild, IDC_TYP4 ), BM_GETCHECK, 0, 0 ) == BST_CHECKED )
		return 256;
	else
		return 0;

}

void CExample5::SetTime( double _time )
{
	char strTmp[16];
	sprintf( strTmp, "%0.3fs", _time );
	SetWindowText( GetDlgItem( timeChild, IDC_TIME ), strTmp );
}

void CExample5::DoFrame( double _timeDelta )
{
	static int oldType = 70;
	int type = GetRadioButton();

	if( oldType != type )
	{
		double time = MakeMandelBrot( buffer, type );
		SetTime( time );
	}

	surface->Blit( GetDD() );

	oldType = type;
}

double CExample5::MakeMandelBrot( dword_ *surf, int max_iterations )
{
	CTimer timer;

	for( int col=0; col<SURF_WIDTH; col++ )
	{
		for( int row=0; row<SURF_HEIGHT; row++ )
		{
			double x = -2 + ( (double)col * 4./SURF_WIDTH );			// -2 -> 2 x-axis values that are important
			double y = -1 + ( (double)row * 2./SURF_HEIGHT );			// -i -> i values that are important in y-axis

			// For each pixel...
			CComplexNumbers *z = new CComplexNumbers( x, y );

			// Start value for c = 1 + i
			CComplexNumbers c( -1., 0.25 );								// A good start value for a mandelbrot set -1 + 0.25i

			for( int i=0; i<max_iterations; i++ )
			{
				// Mandelbrot formula...
				CComplexNumbers Z = *z * *z + c;

				// If the set break into infanity
				if( Z.Modulos() > 4 )
					break;

				// Do y = Z, for the next iteration
				delete z;
				z = new CComplexNumbers( Z );
			}
			delete z;
			z = NULL;

			unsigned char color = max_iterations - i;

			// Do fake coloring....
			surf[ row*SURF_WIDTH + col ] = color*16 << 16 | color*8 << 8 | color;
		}

		// Only for updating the text in upper right corner.
		SetTime( timer.Time() );
	}

	return timer.Time();
}