// Application.cpp: implementation of the CApplication class.
//
//////////////////////////////////////////////////////////////////////

#include "..\include\Application.h"

CApplication::CApplication()
{
	frameTimer	= NULL;
	ddraw		= NULL;
	m_hWnd		= NULL;
	m_bActive	= false;
}

CApplication::~CApplication()
{
}

void CApplication::Init( HINSTANCE _hInstance )
{
	RECT rc; rc.left = 0; rc.top = 0; rc.right = m_width; rc.bottom = m_height;

#ifdef __SE_WINDOWED__
	AdjustWindowRect( &rc, WS_OVERLAPPEDWINDOW, NULL );
#endif

	WNDCLASSEX wcex;
	// fill in some default values
	wcex.cbSize			= sizeof(WNDCLASSEX); 
	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= _hInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor( NULL, IDC_ARROW );
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= m_name.c_str();
	wcex.hIconSm		= NULL;

	if( RegisterClassEx( &wcex ) == 0 )
		assert( 0 );

	m_hWnd = CreateWindow( m_name.c_str(), m_name.c_str(), WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_VISIBLE, CW_USEDEFAULT, CW_USEDEFAULT, rc.right - rc.left, rc.bottom - rc.top, NULL, NULL, _hInstance, NULL );

	m_hInstance = _hInstance;

	// Draw
	ddraw = new CDDraw( m_hWnd, m_width, m_height );
	assert( ddraw );

#ifdef __SE_WINDOWED__
	UpdateRectangles();
#endif

	ShowWindow( m_hWnd, SW_SHOW );
	UpdateWindow( m_hWnd );

	frameTimer = new CTimer();
	assert( frameTimer );


#ifdef __SE_FULLSCREEN__
	m_bActive = true;
#endif

}

#ifdef __SE_WINDOWED__
void CApplication::UpdateRectangles()
{
	if( m_hWnd )
	{
		RECT m_rcViewport;
		GetClientRect( m_hWnd, &m_rcViewport );

		m_rcScreen = m_rcViewport;
		ClientToScreen( m_hWnd, (POINT*)&m_rcScreen.left );
		ClientToScreen( m_hWnd, (POINT*)&m_rcScreen.right );
	}
}
#endif

void CApplication::End()
{
	if( frameTimer )
		delete frameTimer;

	if( ddraw )
		delete ddraw;

	if( m_hWnd )
		DestroyWindow( m_hWnd );

	ApplicationDestroy();
}

void CApplication::Run()
{
	while( 1 )
	{
		double lastFrameTime = frameTimer->Time();
		frameTimer->Reset();

		if( m_bActive )
			DoFrame( lastFrameTime );
		
		MSG msg;
		while( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
		{
			// Translate and dispatch
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}

#ifdef __SE_WINDOWED__
		if( m_bActive )
			ddraw->m_pddsPrimary->Blt( &m_rcScreen, ddraw->m_pddsBackBuffer, NULL, DDBLT_WAIT, NULL );

		Sleep( 1 );
#endif

#ifdef __SE_FULLSCREEN__
		if( m_bActive )
			ddraw->m_pddsPrimary->Flip( NULL, DDFLIP_WAIT );
#endif

	}
}

void CApplication::SceneInit()
{

}

void CApplication::SceneEnd()
{

}

void CApplication::DoFrame( double _timeDelta )
{
	Sleep( 50 );
}