#include "..\include\openSE.h"
#include "..\include\Converter.h"

openSE::openSE()
{
	m_pDDS = NULL;
	buffer = NULL;
	m_width = m_height = 0;
	Convert = NULL;
}

openSE::~openSE()
{
	Close();
}

dword_* openSE::Open( const int _width, const int _height, const CDDraw* _ddraw )
{
	assert( _width > 32 );
	assert( _height > 32 );

	DDSURFACEDESC2 ddsd;
	ZeroMemory( &ddsd, sizeof(ddsd) );
	ddsd.dwSize  = sizeof( ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT | DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN | DDSCAPS_VIDEOMEMORY;
	ddsd.dwWidth = _width;
	ddsd.dwHeight = _height;

	// Create the Direct X Surface
	if( FAILED( _ddraw->m_pDD->CreateSurface( &ddsd, &m_pDDS, NULL ) ) )
		assert( 0 );

	// Enumerate converter
	Convert = GetConverter();
	assert( Convert );

	m_width = _width;
	m_height = _height;

	// Create the openSE surface
	buffer = new dword_[ _width*_height ];

	memset( (void*)buffer, 0x00, sizeof(dword_)*_width*_height );

	return buffer;
}

void openSE::Clear( const dword_ _color )
{
	m_pDDS->Restore();

	ColorClear( (void *)buffer, _color, m_height*m_width );
}

void openSE::Blit( const CDDraw* _ddraw )
{
	_ddraw->m_pddsBackBuffer->Restore();
	m_pDDS->Restore();

	DDSURFACEDESC2 s;
	ZeroMemory( &s, sizeof(DDSURFACEDESC2) );
	s.dwSize = sizeof(DDSURFACEDESC2);

	if( FAILED( m_pDDS->Lock( NULL, &s, DDLOCK_SURFACEMEMORYPTR | DDLOCK_WAIT, NULL ) ) )
	{
		m_pDDS->Unlock( NULL );
		return;
	}

	int src_pitch = m_width*4;
	int dst_pitch = s.lPitch;

	byte_ *src = (byte_*)buffer;
	byte_ *dst = (byte_*)s.lpSurface;

	for( int i=0; i<m_height; i++ )
	{
		Convert( src, dst, m_width );
		src += src_pitch;
		dst += dst_pitch;
	}

	m_pDDS->Unlock( NULL );

	if( FAILED( _ddraw->m_pddsBackBuffer->Blt( NULL, m_pDDS, NULL, DDBLT_WAIT, NULL ) ) )
		assert( 0 );
}

void openSE::Close()
{
	if( m_pDDS )
	{
		m_pDDS->Release();
		m_pDDS = NULL;
	}

	if( buffer )
	{
		delete [] buffer;
		buffer = NULL;
	}
}

CONVERTER openSE::GetConverter()
{
	DDPIXELFORMAT ddpf;
	ZeroMemory( &ddpf, sizeof(ddpf) );
	ddpf.dwSize = sizeof(ddpf);

	if( FAILED( m_pDDS->GetPixelFormat( &ddpf ) ) )
		assert( 0 );

	// check that format is direct color
	if( !(ddpf.dwFlags & DDPF_RGB) )
		assert( 0 );

	// 32bit RGB888 -> 32bit RGB888
	if( ddpf.dwRGBBitCount==32 && ddpf.dwRBitMask==0x00FF0000 && ddpf.dwGBitMask==0x0000FF00 && ddpf.dwBBitMask==0x000000FF)
		return &Convert32_32rgb888;

	// 32bit RGB888 -> 32bit BGR888
	if( ddpf.dwRGBBitCount==32 && ddpf.dwRBitMask==0x000000FF && ddpf.dwGBitMask==0x0000FF00 && ddpf.dwBBitMask==0x00FF0000)
		return &Convert32_32bgr888;

	// 32bit RGB888 -> 24bit RGB888
	if( ddpf.dwRGBBitCount==24 && ddpf.dwRBitMask==0x00FF0000 && ddpf.dwGBitMask==0x0000FF00 && ddpf.dwBBitMask==0x000000FF)
		return &Convert32_24rgb888;

	// 32bit RGB888 -> 24bit BGR888
	if( ddpf.dwRGBBitCount==24 && ddpf.dwRBitMask==0x000000FF && ddpf.dwGBitMask==0x0000FF00 && ddpf.dwBBitMask==0x00FF0000)
		return &Convert32_24bgr888;
    
	// 32bit RGB888 -> 16bit RGB565
	if( ddpf.dwRGBBitCount==16 && ddpf.dwRBitMask==0xF800 && ddpf.dwGBitMask==0x07E0 && ddpf.dwBBitMask==0x001F)
		return &Convert32_16rgb565;

	// 32bit RGB888 -> 16bit BGR565
	if( ddpf.dwRGBBitCount==16 && ddpf.dwRBitMask==0x001F && ddpf.dwGBitMask==0x07E0 && ddpf.dwBBitMask==0xF800)
		return &Convert32_16bgr565;

	// 32bit RGB888 -> 16bit RGB555
	if( ddpf.dwRGBBitCount==16 && ddpf.dwRBitMask==0x7C00 && ddpf.dwGBitMask==0x03E0 && ddpf.dwBBitMask==0x001F)
		return &Convert32_16rgb555;

	// 32bit RGB888 -> 16bit BGR555
	if( ddpf.dwRGBBitCount==16 && ddpf.dwRBitMask==0x001F && ddpf.dwGBitMask==0x03E0 && ddpf.dwBBitMask==0x7C00)
		return &Convert32_16bgr555;

	return 0;
}
